-- Copyright
-- Absorber
-- Author: Steven Bartels
-- Date: 07.07.2015

-- Addon Ref
local Absorber = LibStub("AceAddon-3.0"):NewAddon("Absorber", "AceConsole-3.0","AceEvent-3.0")

-- Globals
local SpellID = 0
local G_ID = 115295         -- Guard
local LC_ID = 116849        -- Life Cocoon
local SS_ID = 65148         -- SacredShield
local DA_ID = 47753         -- Divine Aegis
local PWS_ID = 17           -- Powerword:Shield
local SpS_ID = 114908       -- Spirit Shell
local SB_ID = 112048        -- Shield Barrier

local TotalShield = 0
local BuffAbsorb = 0

local MaxShield = 25000
local Locked = true
local IsVisible = true
local xPos = 200
local yPos = 100
local Width = 100
local Height = 25
local TextScale = 1
local TextPosX = 0
local TextPosY = 0

local defaults = {
  profile = {
    MaxShield = 25000,
    Locked = true,
    IsVisible = true,
    x = 200,
    y = 100,
    Width = 100,
    Height = 25,
    TextScale = 1,
    TextPosX = 0,
    TextPosY = 0,
  }
}

-- Ace OnInitialize
function Absorber:OnInitialize()
    DataBase = LibStub("AceDB-3.0"):New("Absorber", defaults)
 end

-- Ace OnEnable
function Absorber:OnEnable()
    if DataBase.char.Locked == nil or 
       DataBase.char.IsVisible == nil or 
       DataBase.char.MaxShield == nil or
       DataBase.char.x == nil or
       DataBase.char.y == nil or
       DataBase.char.Width == nil or
       DataBase.char.Height == nil or
       DataBase.char.TextScale == nil or
       DataBase.char.TextPosX == nil or
       DataBase.char.TextPosY == nil then
       Absorber:SaveDataBase()
    else
       Absorber:LoadDataBase()
    end

    AbsorberUI:CreateAbsorbBar()
    Absorber:RegisterEvent("UNIT_AURA", "PerformAuraCheck")
    Absorber:RegisterChatCommand("absorber", "SlashCommandFunc")

end

-- Ace OnDisable
function Absorber:OnDisable()
    Absorber:UnregisterEvent("UNIT_AURA")
    Absorber:UnregisterChatCommand("absorber")
end

-- Catching /Commands
function Absorber:SlashCommandFunc(input)
    if input == "help" then
        Absorber:ShowHelp()
    elseif input == "lock" then
        SetLocked(true)        
        Absorber:Print("Absorber is locked now!") 
    elseif input == "unlock" then
        SetLocked(false)    
        Absorber:Print("Absorber is unlocked now!")
    elseif input == "show" and IsVisible == false then
        SetVisibility(true)   
        Absorber:Print("Absorber is visible now!")
    elseif input == "hide" and IsVisible == true then
        SetVisibility(false)    
        Absorber:Print("Absorber is hidden now!")
    elseif input == "config" then
        AbsorberUI:OpenOptions()
    elseif input == "reset" then
        Absorber:ResetValues()
    end
end

-- Performs an update of the UI
function Absorber:PerformUpdate()
    UpdateBar()
end

-- Simple Calculation for total shield
function Absorber:CalculateShield(currentValue)
    if  (currentValue ~= nil ) then
		TotalShield = TotalShield + currentValue
	end
    Absorber:PerformUpdate()
end

-- Check of aura/buff changes
function Absorber:PerformAuraCheck(arg1, arg2)
    if arg2 == "player" then
       TotalShield = 0
       for i=1,25 do
			_, _, _, _, _, _, _, _, _, _, SpellID, _, _, _,BuffAbsorb = UnitBuff("Player", i)
            if SpellID == SS_ID or              -- Sacred Shield
               SpellID == DA_ID or              -- Divine Aegis
               SpellID == PWS_ID or             -- Powerword:Shield
               SpellID == SpS_ID or             -- Spirit Shell
               SpellID == G_ID or               -- Guard
               SpellID == LC_ID or              -- Life Cocoon
               SpellID == SB_ID then            -- Shield Barrier
               Absorber:CalculateShield(BuffAbsorb)
            elseif SpellID == nil then
                break
            end
       end
       Absorber:PerformUpdate()
    end   
end

-- Help
function Absorber:ShowHelp()
     Absorber:Print("Absorber commands:")  
     Absorber:Print("/absorber   help - Showing the help")
     Absorber:Print("/absorber config - Opens the options")
     Absorber:Print("/absorber   lock - Locks the bar") 
     Absorber:Print("/absorber unlock - Unlocks the bar")  
     Absorber:Print("/absorber   show - Makes the bar visible")  
     Absorber:Print("/absorber   hide - Makes the bar invisible")
     Absorber:Print("/absorber  reset - Resets all value to default settings")
     
     Absorber:Print("by Kaldarus <Call for Sanity> on Kult der Verdammten")
end

-- Visibilty
function GetVisibility()
    return IsVisible
end

function SetVisibility(value)
    IsVisible = value
    Absorber:SaveDataBase()
    Absorber:PerformUpdate()
end

function Absorber:SaveDataBase()
    DataBase.char.Locked = Locked
    DataBase.char.IsVisible = IsVisible
    DataBase.char.MaxShield = MaxShield
    DataBase.char.x = xPos
    DataBase.char.y = yPos
    DataBase.char.Height = Height
    DataBase.char.Width = Width
    DataBase.char.TextScale = TextScale
    DataBase.char.TextPosY = TextPosY
    DataBase.char.TextPosX = TextPosX
    --Absorber:Print("Save complete!")
end

function Absorber:LoadDataBase()
    Locked = DataBase.char.Locked
    IsVisible = DataBase.char.IsVisible
    MaxShield = DataBase.char.MaxShield
    xPos = DataBase.char.x 
    yPos = DataBase.char.y
    Height = DataBase.char.Height
    Width = DataBase.char.Width
    TextScale = DataBase.char.TextScale
    TextPosY = DataBase.char.TextPosY
    TextPosX = DataBase.char.TextPosX
    --Absorber:Print("Load complete!")
end

function SetPos(xValue, yValue)
    xPos = xValue
    yPos = yValue
    Absorber:SaveDataBase()
end

function GetPos()
    return xPos, yPos
end

function GetTotalShield()
    return TotalShield
end

function GetMaxShield()
    return MaxShield
end

function SetMaxShield(value)
    MaxShield = value
    Absorber:SaveDataBase()
end

function GetHeight()
    return Height
end

function GetWidth()
    return Width
end


function SetHeight(value)
    Height = value
    Absorber:SaveDataBase()
    SetBarHeight(Height)
end

function SetWidth(value)
    Width = value
    Absorber:SaveDataBase()
    SetBarWidth(Width)
end

-- Lock/Unlock
function GetLocked()
    return Locked
end

function SetLocked(value)
    Locked = value
    Absorber:SaveDataBase()
end

function SetTextScale(value)
    TextScale = value;
    Absorber:SaveDataBase()
    SetBarTextScale(TextScale)
end

function GetTextScale()
    return TextScale
end

function SetTextPosX(value)
    TextPosX = value;
    Absorber:SaveDataBase()
    SetBarTextPositionX(TextPosX)
end

function SetTextPosY(value)
    TextPosY = value;
    Absorber:SaveDataBase()
    SetBarTextPositionY(TextPosY)
end

function GetTextPos()
    return TextPosX, TextPosY
end

function Absorber:ResetValues()
    Locked = defaults.profile.Locked
    IsVisible = defaults.profile.IsVisible
    MaxShield = defaults.profile.MaxShield
    xPos = defaults.profile.x 
    yPos = defaults.profile.y
    Height = defaults.profile.Height
    Width = defaults.profile.Width
    TextScale = defaults.profile.TextScale
    TextPosX = defaults.profile.TextScale
    TextPosY = defaults.profile.TextScale
    ResetBar()
    Absorber:SaveDataBase()
    Absorber:PerformUpdate()  
    Absorber:Print("Reset complete!")
end