-- Copyright
-- Absorber
-- Author: Steven Bartels
-- Date: 07.07.2015    

AbsorberUI = LibStub("AceGUI-3.0")
local Media = LibStub("LibSharedMedia-3.0")
Media:Register("background", "Absorb_DefaultBG", "Interface\\Addons\\Absorber\\Images\\BarDefaultBlack.tga")
Media:Register("background", "Absorb_DefaultFG", "Interface\\Addons\\Absorber\\Images\\BarDefaultGreen.tga")

-- Options
local OptionsOpen = false

-- Bar
local BarContainer = AbsorberUI:Create("SimpleGroup")
local BarForground = AbsorberUI:Create("Label")
local BarBackground = AbsorberUI:Create("Label")
local ValueLabel = AbsorberUI:Create("Label")



function AbsorberUI:OpenOptions()
    if not OptionsOpen then
        local Options = AbsorberUI:Create("Frame")        
        Options:SetTitle("Absorber Options")
        Options:SetStatusText("by Kaldarus <Call for Sanity> - Kult der Verdammten")
        Options:SetLayout("Flow")    
        Options:SetCallback("OnClose", function(widget) AbsorberUI:Release(widget) OptionsOpen = false end)
        Options:SetPoint("Center",0,0)
        Options:EnableResize(false)
        Options:SetHeight(300)
        Options:SetWidth(550)

        local BarGroup = AbsorberUI:Create("InlineGroup")
        BarGroup:SetTitle("Bar Settings")
        BarGroup:SetAutoAdjustHeight(false)
        BarGroup:SetLayout("List")
        BarGroup:SetHeight(200)
        BarGroup:SetWidth(250)
        Options:AddChild(BarGroup)

        -- Absorb Textbox
        local Editbox = AbsorberUI:Create("EditBox")
        Editbox:SetLabel("Max Aborb Amount Shown")
        Editbox:SetText(GetMaxShield())
        Editbox:SetWidth(200)
        Editbox:DisableButton(true)
        Editbox:SetCallback("OnEnterPressed", function(widget) SetMaxShield(Editbox.editbox:GetNumber()) AbsorberUI:ClearFocus() end)
        Editbox.editbox:SetNumeric()
        BarGroup:AddChild(Editbox)

        -- Width and Hight Slider
        local WidthSlider = AbsorberUI:Create("Slider")
        WidthSlider:SetLabel("Bar Width")
        WidthSlider:SetValue(GetWidth())
        WidthSlider:SetSliderValues(10,500,5)
        WidthSlider:SetWidth(200)
        WidthSlider:SetCallback("OnValueChanged", function(widget) SetBarWidth(WidthSlider.slider:GetValue()) end)
        WidthSlider:SetCallback("OnMouseUp", function(widget) SetWidth(WidthSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        BarGroup:AddChild(WidthSlider)

        local HeightSlider = AbsorberUI:Create("Slider")
        HeightSlider:SetLabel("Bar Height")
        HeightSlider:SetValue(GetHeight())
        HeightSlider:SetSliderValues(10,500,5)
        HeightSlider:SetWidth(200)
        HeightSlider:SetCallback("OnValueChanged", function(widget) SetBarHeight(HeightSlider.slider:GetValue()) end)
        HeightSlider:SetCallback("OnMouseUp", function(widget) SetHeight(HeightSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        BarGroup:AddChild(HeightSlider)

         -- Unlock/Lock CheckBox
        local LockCheckbox = AbsorberUI:Create("CheckBox")
        LockCheckbox:SetLabel("Unlock/Lock")
        LockCheckbox:SetValue(true)
        LockCheckbox:SetCallback("OnValueChanged", function(widget) SetLocked(LockCheckbox:GetValue()) end)
        BarGroup:AddChild(LockCheckbox)

        local TextGroup = AbsorberUI:Create("InlineGroup")
        TextGroup:SetTitle("Text Settings")
        TextGroup:SetLayout("List")
        TextGroup:SetAutoAdjustHeight(false)
        TextGroup:SetHeight(200)
        TextGroup:SetWidth(250)
        Options:AddChild(TextGroup)

        -- Text Size Slider
        local TextScaleSlider = AbsorberUI:Create("Slider")
        TextScaleSlider:SetLabel("Text Size")
        TextScaleSlider:SetValue(GetTextScale())
        TextScaleSlider:SetSliderValues(0.1,5,0.1)
        TextScaleSlider:SetWidth(200)
        TextScaleSlider:SetCallback("OnValueChanged", function(widget) SetBarTextScale(TextScaleSlider.slider:GetValue()) end)
        TextScaleSlider:SetCallback("OnMouseUp", function(widget) SetTextScale(TextScaleSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        TextGroup:AddChild(TextScaleSlider)

        local TextX, TextY = GetTextPos()

        -- Text X Slider
        local TextPosXSlider = AbsorberUI:Create("Slider")
        TextPosXSlider:SetLabel("Relative Text X Position")
        TextPosXSlider:SetValue(TextX)
        TextPosXSlider:SetSliderValues(-100,100,1)
        TextPosXSlider:SetWidth(200)
        TextPosXSlider:SetCallback("OnValueChanged", function(widget) SetBarTextPositionX(TextPosXSlider.slider:GetValue()) end)
        TextPosXSlider:SetCallback("OnMouseUp", function(widget) SetTextPosX(TextPosXSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        TextGroup:AddChild(TextPosXSlider)

        -- Text Y Slider
        local TextPosYSlider = AbsorberUI:Create("Slider")
        TextPosYSlider:SetLabel("Relative Text Y Position")
        TextPosYSlider:SetValue(TextY)
        TextPosYSlider:SetSliderValues(-100,100,1)
        TextPosYSlider:SetWidth(200)
        TextPosYSlider:SetCallback("OnValueChanged", function(widget) SetBarTextPositionY(TextPosYSlider.slider:GetValue()) end)
        TextPosYSlider:SetCallback("OnMouseUp", function(widget) SetTextPosY(TextPosYSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        TextGroup:AddChild(TextPosYSlider)
    end
end

function AbsorberUI:CreateAbsorbBar()
    BarContainer:SetLayout("FULL")
    BarContainer.frame:SetPoint("CENTER",DataBase.char.x, DataBase.char.y)
    BarContainer.frame:SetSize(GetWidth(),GetHeight())
    BarContainer.frame:SetMovable(true)

    BarBackground:SetImage(Media:Fetch("background", "Absorb_DefaultBG"))
    BarBackground:SetImageSize(GetWidth(),GetHeight())
    BarBackground.frame:SetMovable(true)   
    BarBackground.frame:SetFrameStrata("BACKGROUND") 
    BarBackground.frame:SetScript("OnMouseDown",BarOnMouseDown)
    BarBackground.frame:SetScript("OnMouseUp",BarOnMouseUp)
    BarBackground:SetPoint("LEFT",BarContainer.frame,0, 0)
    BarContainer:AddChild(BarBackground)

    BarForground:SetImage(Media:Fetch("background", "Absorb_DefaultFG"))
    BarForground:SetImageSize(0,GetHeight())
    BarForground.frame:SetMovable(true)
    BarForground.frame:SetFrameStrata("MEDIUM")
    BarForground.frame:SetPoint("LEFT",BarContainer.frame, 0, 0)
    BarContainer:AddChild(BarForground)

    
    local TextX, TextY = GetTextPos()

    ValueLabel:SetText(GetTotalShield())
    ValueLabel.frame:SetScale(GetTextScale())
    ValueLabel.frame:SetMovable(true)
    ValueLabel.frame:SetFrameStrata("High")
    ValueLabel.label:SetJustifyH("CENTER")
	ValueLabel.label:SetJustifyV("CENTER")
    ValueLabel.frame:SetPoint("CENTER", TextX, TextY)

    BarContainer:AddChild(ValueLabel)

    UpdateBar()
end

-- Events
function BarOnMouseDown(this, button)
    if button == "LeftButton" and not GetLocked() then
		BarContainer.frame:StartMoving()  
    end
end

function BarOnMouseUp(this, button)
    local xPos = 0;
    local yPos = 0;
    if button == "LeftButton" and not GetLocked() then
		BarContainer.frame:StopMovingOrSizing()    
        _, _, _, xPos, yPos = BarContainer:GetPoint()
    SetPos(xPos,yPos)
    end
end

function UpdateBar()
    local xPos, yPos
    local barvalue = (GetTotalShield() / GetMaxShield())
    local barwidth = barvalue * GetWidth()

    if barwidth <= 1 then
        BarForground.frame:Hide()
    else 
        _,_,_,xPos,yPos = BarBackground:GetPoint()
        BarForground.frame:Show()
        BarForground:SetImageSize(barwidth,GetHeight())
        BarForground.frame:SetPoint("LEFT", xPos -(GetWidth() - barwidth)/2,yPos)
    end
    ValueLabel:SetText(GetTotalShield())
    SetBarVisibility()
end

function SetBarVisibility()
    if DataBase.char.IsVisible then
        BarContainer.frame:Show()
    elseif not DataBase.char.IsVisible then
        BarContainer.frame:Hide()
    end
end

function SetBarWidth(value)
    BarContainer.frame:SetSize(value,GetHeight())
    BarBackground:SetImageSize(value,GetHeight())
    UpdateBar()
end

function SetBarHeight(value)
    BarContainer.frame:SetSize(GetWidth(),value)
    BarBackground:SetImageSize(GetWidth(),value)
    UpdateBar()
end

function SetBarTextScale(value)
    ValueLabel.frame:SetScale(value)
end

function SetBarTextPositionX(value)
    local _, TextY = GetTextPos()
    ValueLabel.frame:SetPoint("CENTER", value, TextY)
end


function SetBarTextPositionY(value)
    local TextX, _ = GetTextPos()
    ValueLabel.frame:SetPoint("CENTER", TextX, value)
end


function ResetBar()
    BarContainer.frame:SetSize(GetWidth(),GetHeight())
    BarBackground:SetImageSize(GetWidth(),GetHeight())
    BarForground:SetImageSize(0,GetHeight())
    
    BarContainer.frame:SetPoint("LEFT",DataBase.char.x, DataBase.char.y)
    BarBackground:SetPoint("LEFT",BarContainer.frame,0, 0)
    BarForground.frame:SetPoint("LEFT",BarContainer.frame, 0, 0)
    ValueLabel.frame:SetPoint("CENTER", 0, 0)
end